(* 
    File:        IntervalX.thy
    Theory Name: IntervalX
    Logic Image: Integ
*)

IntervalX = Time +

datatype bound =  Val   int ("# _") |
                  Pinf  ("+inf") |
                  Minf  ("-inf") |
                  Next  tbool ("++ _") |
                  Prev  tbool ("-- _") 
datatype interval = oo  bound bound ("<_,_>") |
                    cc  bound bound ("[_,_]") |
                    oc  bound bound ("<_,_]") |
                    co  bound bound ("[_,_>") |
                    sng bound ("[_]")
  
consts
  win      :: [int, interval] => tbool
  after    :: [int, bound] => tbool
  before   :: [int, bound] => tbool
  sublft   :: [interval, int] => interval
  subrgt   :: [interval, int] => interval
  eq       :: [interval, interval] => bool (infixr 20)
  
primrec
  after_val  "(after x (# v))  = const (v <= x)"
  after_pinf "(after x (+inf)) = false"
  after_minf "(after x (-inf)) = true"
  after_next "(after x (++ N)) =
	        (%v. (? t. #0 < t & t <= x & N (v+t) &
	                (! t'. (#0 < t' & t'< t) --> ~N(v+t'))))" 
  after_prev "(after x (-- P)) =
	        (%v. (? t. t < #0  & t <= x & P (v+t) &
			(! t'. (t < t' & t'< #0) --> ~P(v+t'))) |
		     (! t'. t'<#0 --> ~P(v+t')))" 
primrec
  before_val  "(before x (# v))  = const (x <= v)"
  before_pinf "(before x (+inf)) = true"
  before_minf "(before x (-inf)) = false"
  before_next "(before x (++ N)) =
	        (%v. (? t. #0 < t & x <= t & N (v+t) &
			(! t'. (#0 < t' & t'< t) --> ~N(v+t'))) |
		     (! t'. #0<t' --> ~N(v+t')))" 
  before_prev "(before x (-- P)) =
	        (%v. ? t. t < #0  & x <= t & P (v+t) &
		       (! t'. (t < t' & t'< #0) --> ~P(v+t')))" 
primrec
  win_cc  "(win v [l,u]) = ((after v l) #& (before v u))"
  win_oo  "(win v <l,u>) = ((after (v-#1) l) #& (before (v+#1) u))"
  win_oc  "(win v <l,u]) = ((after (v-#1) l) #& (before v u))"
  win_co  "(win v [l,u>) = ((after v l) #& (before (v+#1) u))"
  win_sng "(win v [b])   = ((after v b) #& (before v b))"  

primrec
  sublft_cc  "(sublft [l,u] v) = [l,#v>"
  sublft_oo  "(sublft <l,u> v) = <l,#v>"
  sublft_oc  "(sublft <l,u] v) = <l,#v>"
  sublft_co  "(sublft [l,u> v) = [l,#v>"
  sublft_sng "(sublft [b] v)   = [b,#v>"

primrec
  subrgt_cc  "(subrgt [l,u] v) = <#v,u]"
  subrgt_oo  "(subrgt <l,u> v) = <#v,u>"
  subrgt_oc  "(subrgt <l,u] v) = <#v,u]"
  subrgt_co  "(subrgt [l,u> v) = <#v,u>"
  subrgt_sng "(subrgt [b] v)   = <#v,b]"

defs
  eq_def "(i1 eq i2) == (!x. (win x i1)=(win x i2))"

end
