(* 
    File:        TilcoX.thy
    Theory Name: TilcoX
    Logic Image: Integ
*)

TilcoX = IntervalX +
consts
  at       :: [tbool, interval] => tbool                  ("_ @ _"  [36,37] 36)
  happen   :: [tbool, interval] => tbool                  ("_ ? _"  [36,37] 36)
  happen1  :: [tbool, interval] => tbool                  ("_ ?! _" [36,37] 36)

  "->>"    :: [tbool, tbool] => tbool                               (infixr 25)
  "-<<"    :: [tbool, tbool] => tbool                               (infixr 25)
  ":="     :: ['a tfun, 'a tfun] => tbool                           (infixr 38)
  until    :: [tbool, tbool] => tbool
  since    :: [tbool, tbool] => tbool

  up       :: tbool => tbool
  down     :: tbool => tbool
  tinv      :: ('a tfun) => tbool
  rule     :: tbool => bool
  fact     :: tbool => bool

syntax
  "&&"     :: [interval, interval] => interval                      (infixr 40)
  "||"     :: [interval, interval] => interval                      (infixr 40)
  "@tEval":: 'a tfun => 'a tfun ("tEval[_]")
translations
  "A @   (i &&  j)"  => "(A @   i) #& (A @   j)"
  "A ?   (i &&  j)"  => "(A ?   i) #& (A ?   j)"
  "A ?!  (i &&  j)"  => "(A ?!  i) #& (A ?!  j)"
  "A @   (i ||  j)"  => "(A @   i) #| (A @   j)"
  "A ?   (i ||  j)"  => "(A ?   i) #| (A ?   j)"
  "A ?!  (i ||  j)"  => "(A ?!  i) #| (A ?!  j)"
  
defs
  at_def       "A @  i       == (%x. ! t. (win t i) x --> A(x+t))"
  happen_def   "A ?  i       == (%x. ? t. (win t i) x & A(x+t))"
(*
  happen1_def  "A ?! i       == (%x. ?! t. t:i & A(t+x))"
 *)
  Tnext_def    "A ->> B      == A #--> (B @ [##1])" 
  Tprev_def    "A -<< B      == A #--> (B @ [##-1])"
  Tassign_def  "A := exp     == (%x. A x = exp (x+#-1))"
  until_def    "until P Q    == Q @ <##0,++P>"
  since_def    "since P Q    == Q @ <--P,##0>"
  up_def       "up A         == (%t. ~A(t+#-1) &  A t)"
  down_def     "down A       == (%t.  A(t+#-1) & ~A t)"
  tinv_def     "tinv A       == (%x.  A x = (A (x+#-1)) )"
  rule_def     "rule A       == (! t. now A t)"
  fact_def     "fact A       == (? t. now A t)"

end
ML

  fun tEval_tr [Const(c,t) $ term1 $ term2 ] =
       Const("binary",dummyT) $ Const(c,t) $ tEval_tr[term1] $ tEval_tr[term2]
    | tEval_tr [Free(v,t) $ term1 $ term2 ] =
       Const("binary",dummyT) $ Free(v,t) $ tEval_tr[term1] $ tEval_tr[term2]
    | tEval_tr [Const(c,t) $ term1] =
       Const("unary",dummyT) $ Const(c,t) $ tEval_tr[term1]
    | tEval_tr [Free("const",t) $ term1] =
       Free("const",t) $ term1 
    | tEval_tr [Free("var",t) $ term1] =
        Free("var",t) $term1 
    | tEval_tr [Free(v,t) $ term1] =
       Const("unary",dummyT) $ Free(v,t) $ tEval_tr[term1] 
    | tEval_tr [Const(c,t)] = Const("const",dummyT) $ Const(c,t)
    | tEval_tr [ a $ b] = 
       let
         exception exTEvalAbs;
         fun tEvalAbs_tr[a $ b] = tEvalAbs_tr[a] $ tEval_tr[b]
           | tEvalAbs_tr[Abs(v,t,b)] = Abs(v,t,tEval_tr[b])
           | tEvalAbs_tr ts = raise exTEvalAbs
       in
         tEvalAbs_tr[a $ b] handle exTEvalAbs => a $ b
       end
    | tEval_tr [Abs(v,t,b)] = Abs(v,t,tEval_tr[b])
    | tEval_tr [term] = term
    | tEval_tr ts = raise TERM ("tEval_tr", ts);
  val parse_translation = [("@tEval",tEval_tr)];

  local
    fun tEval_tr' [Const("binary",t) $ f $ a $ b] =
        f $ tEval_tr'[a] $ tEval_tr'[b]
      | tEval_tr' [Const("unary",t) $ f $ a]=
         f $ tEval_tr'[a]
      | tEval_tr' [Const("const",t) $ Const(c,t2) ]= Const(c,t2)
      | tEval_tr' [Const("var",t) $ Const(c,t2) ]= Const(c,t2)
      | tEval_tr' [t] = t
      | tEval_tr' ts = raise Match;
  in
    fun tEvalBin_tr' [Const(c,t) , a , b] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a] $ tEval_tr'[b])
      | tEvalBin_tr' [Const(c,t) , a , b , d] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a] $ tEval_tr'[b]) $d
      | tEvalBin_tr' [Free(v,t) , a , b] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a] $ tEval_tr'[b])
      | tEvalBin_tr' [Free(v,t) , a , b , c] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a] $ tEval_tr'[b]) $c
      | tEvalBin_tr' [t] = t
      | tEvalBin_tr' ts = raise Match;

    fun tEvalUna_tr' [Const(c,t) , a] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a])
      | tEvalUna_tr' [Const(c,t) , a, b] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a]) $ b
      | tEvalUna_tr' [Free(v,t) , a] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a])
      | tEvalUna_tr' [Free(v,t) , a, b] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a]) $ b
      | tEvalUna_tr' [t] = t
      | tEvalUna_tr' ts = raise Match;
  end
  val print_translation = [("binary",tEvalBin_tr'),("unary",tEvalUna_tr')];
