(* 
    File:	 Time.thy
    Theory Name: Time
    Logic Image: Integ
*)

Time  =  IntArith +
types
  time    = int 
  tbool   = (time => bool)
  'a tfun = (time => 'a)
consts
  T        :: time
  now      :: [tbool, time] => bool
  true     :: tbool
  false    :: tbool

  "#~"     :: tbool => tbool                                   ("#~ _" [40] 40)
  "#&"     :: [tbool, tbool] => tbool                               (infixr 35)
  "#|"     :: [tbool, tbool] => tbool                               (infixr 30)
  "#="     :: ['a tfun, 'a tfun] => tbool                           (infixr 25)
  "#~="    :: [tbool, tbool] => tbool                               (infixr 33)
  "#-->"   :: [tbool, tbool] => tbool                               (infixr 25)

  TIF      :: [tbool, 'a tfun, 'a tfun] => ('a tfun)
                                            ("(IF (_)/ THEN (_)/ ELSE (_))" 10)


  TAll     :: ('a => tbool) => tbool                         (binder "#! "  10)
  TEx      :: ('a => tbool) => tbool                         (binder "#? "  10)
  TEx1     :: ('a => tbool) => tbool                         (binder "#?! " 10)
  TBall    :: ['a set, 'a => tbool] => tbool
  TBex     :: ['a set, 'a => tbool] => tbool

  const    :: 'a => ('a tfun)
  unary    :: ['a => 'b, 'a tfun] => ('b tfun)
  binary   :: [['a,'b] => 'c,'a tfun,'b tfun] => ('c tfun)

syntax
  "*TBall" :: [idts,'a set,tbool]=>tbool  ("(3#! _:_./ _)" 10)
  "*TBex"  :: [idts,'a set,tbool]=>tbool  ("(3#? _:_./ _)"  10)

translations
  "A #~= B"          == "#~ (A #= B)"
  "#! x:A. P"        == "TBall A (%x. P)"
  "#? x:A. P"        == "TBex A (%x. P)"

defs
  now_def      "now A t      == A(T+t)"

  Ttrue_def    "true         == (%x. True)"
  Tfalse_def   "false        == (%x. False)"

  Tnot_def     "#~ A         == (%x. ~ A x)"
  Tand_def     "A #& B       == (%x. A x & B x)"
  Tor_def      "A #| B       == (%x. A x | B x)"
  Tequiv_def   "A #= B       == (%x. A x = B x)"
  Timply_def   "A #--> B     == (%x. A x --> B x)"

  TIF_def      "TIF P x y    == (%z. if (P z) then (x z) else (y z))"

  TAll_def     "TAll P       == (%y. ! x. P x y)"
  TEx_def      "TEx P        == (%y. ? x. P x y)"
  TEx1_def     "TEx1 P       == (%z. ?! x. P x z)"
  TBall_def    "TBall A P    == (%y. ! x:A. P x y)"
  TBex_def     "TBex A P     == (%y. ? x:A. P x y)"
  
  const_def    "const K      == (%x. K)"
  unary_def    "unary f B    == (%t. f (B t))"
  binary_def   "binary f A B == (%t. f (A t) (B t))"
end

